@@ -2,7 +2,7 @@
 /* Copyright (C) 2007 Rahul Siddharthan     */
 /* Licensed under the GNU GPL, version 2    */
 /* See file COPYING in source distribution  */
-
+ /* This is the working folder for sigma - Gayathri */
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
@@ -12,6 +12,9 @@
 #include "fasta.h"
 #include "commonroutines.h"
 
+    double *bmfmat;
+    int *bmlmat,*bmmmat;
+
 void printn(char *s, int n) 
 {
     int m;
@@ -56,76 +59,140 @@ void printfrags(params *v)  /* debugging routine */
 
 void init_sw_matrix(params *v, GArray *seqs) 
 {
-    int n,maxlen=0;
-    fastaseq *seq;
+    int n,i,maxlen=0;
+    fastaseq *seq; int j=0;
     
     for (n=0; n<seqs->len; n++) {
         seq= &g_array_index(seqs,fastaseq,n);
         if (seq->seq->len > maxlen)
             maxlen=seq->seq->len;
     }
-    if (maxlen > v->maxfraglen)
-        maxlen = v->maxfraglen;
-    swmat.fmat = malloc((maxlen+1)*(maxlen+1)*sizeof(double));
-    swmat.lmat = malloc((maxlen+1)*(maxlen+1)*sizeof(int));
-    swmat.mmat = malloc((maxlen+1)*(maxlen+1)*sizeof(int));
+    swmat.fmat = malloc((maxlen+1)*sizeof(double));
+    swmat.lmat = malloc((maxlen+1)*sizeof(int));
+    swmat.mmat = malloc((maxlen+1)*sizeof(int));
+    for (i=0; i<=maxlen; i++) {
+        swmat.fmat[i]=1.0;
+        swmat.lmat[i]=0;
+        swmat.mmat[i]=0;
+    }
+
     swmat.m = maxlen+1;
     swmat.n = maxlen+1;
+
+bmfmat =calloc (sizeof(double),maxlen+1);
+bmlmat = calloc(sizeof(int), maxlen+1);
+bmmmat = calloc(sizeof(int), maxlen+1); 
+    for (j = 0;j<maxlen;j++)
+    {
+	bmfmat[j] = 1.0;
+	bmlmat[j] = 0;
+	bmmmat[j] = 0;
+    }	
+
 }
 
 
+
 void best_match(params *v, seq_frag *frag1, seq_frag *frag2,
-                int *n1, int *n2, int *lbestout, double *bestout) 
+                int *n1, int *n2, int *lbestout, double *bestout)
 {
     int sl1, sl2, bestm=0, bestn=0, lbest=0, m, n, m_old, l_new, m_new;
     double best=1.0, f_old, f_new;
     char a1, b1;
+    double lfmat = 0.0;
+    int i, j =0;
+    int llmat = 0, lmmat = 0;
 
     sl1 = frag1->seqlen;
-    sl2 = frag2->seqlen;
-    
-    for (m=0; m<=sl1; m++) 
-        for (n=0; n<=sl2; n++) {
-            swmat.fmat[m*sl2+n]=1.0;
-            swmat.lmat[m*sl2+n]=0;
-            swmat.mmat[m*sl2+n]=0;
-    }
-    
-
+    sl2 = frag2->seqlen; 
+    for (i=0; i<=sl2; i++) {
+        swmat.fmat[i]=1.0;
+        swmat.lmat[i]=0;
+        swmat.mmat[i]=0;
+	}
+    for (j = 0;j<sl2;j++)
+    {
+	bmfmat[j] = 1.0;
+	bmlmat[j] = 0;
+	bmmmat[j] = 0;
+	}		
     for (m=1; m<=sl1; m++)
+    {
         for (n=1; n<=sl2; n++) {
             a1 = frag1->aseq[m-1];
             b1 = frag2->aseq[n-1];
-            f_old = swmat.fmat[(m-1)*sl2+(n-1)];
-            m_old = swmat.mmat[(m-1)*sl2+(n-1)];
-            l_new = swmat.lmat[(m-1)*sl2+(n-1)] + 1;
+
+            f_old = swmat.fmat[n-1];
+            m_old = swmat.mmat[n-1];
+            l_new = swmat.lmat[n-1] + 1;
+
             if ((a1==b1)&&(a1 != 'N')) {
                 m_new = m_old;
-                f_new = f_old * sqrt(frag1->abgw[m-1]*frag2->abgw[n-1])
+                f_new = f_old * sqrt(frag1->abgw[m-1]*frag2->abgw[n-1]) 
                     * (float) (l_new) / ((float)(l_new - m_old));
+                   
             }
             else {
                 m_new = m_old + 1;
                 f_new = f_old * l_new/(m_old + 1.0);
             }
-            if (f_new >= 1.0) {
+            if (f_new >= 1.0) 
+	    {
                 f_new = 1.0;
                 l_new = 0;
                 m_new = 0;
             }
-            swmat.fmat[m*sl2+n] = f_new;
-            swmat.lmat[m*sl2+n] = l_new;
-            swmat.mmat[m*sl2+n] = m_new;
+
+            bmfmat[n] = f_new; 
+            bmlmat[n] = l_new;
+            bmmmat[n] = m_new;
+
             if (f_new < best) {
                 best = f_new;
                 bestm = m;
                 bestn = n;
             }
-        }
-    while ((swmat.fmat[(bestm-lbest)*sl2+(bestn-lbest)] < 1.0)
-           && (lbest < bestm) && (lbest < bestn))
-        lbest++;
+        } /* end of n loop */
 
+	memcpy(swmat.fmat,bmfmat,sizeof(double)* (sl2+1));
+	memcpy(swmat.lmat,bmlmat,sizeof(int)*(sl2+1));
+	memcpy(swmat.mmat,bmmmat,sizeof(int)*sl2);
+    } /*end of m loop */
+ i = bestm; j=bestn;
+      if (i>j) 
+	{
+		i=i-j; j=0;
+	}
+	else if (j>i)
+	{
+		j=j-i; i= 0;
+}
+else
+ i=j=0;
+    f_old = 1.0; l_new = 1; m_old = 0;
+    while ((i< bestm) && (j<bestn))
+    {
+            a1 = frag1->aseq[i];
+            b1 = frag2->aseq[j];
+	            if ((a1==b1)&&(a1 != 'N')) {
+	                f_new = f_old * sqrt(frag1->abgw[i]*frag2->abgw[j])  
+        	            * (float) (l_new) / ((float)(l_new - m_old));
+	            }
+        	    else {
+        	        f_new = f_old * l_new/(m_old + 1.0);
+			m_old ++;
+
+		    }
+		    if (f_new < 1.0)
+		    {
+			    lbest++;
+		    }
+		    else 
+		    {
+			    lbest = 0; l_new =0; f_new = 1.0; m_old = 0;
+		    }
+		    i++; j++;f_old = f_new; l_new++;
+    } 
     best = best * (sl1-lbest+1) * (sl2-lbest+1);            
     if (best < v->pmatch) {
         *n1 = bestm-lbest;
@@ -142,7 +209,6 @@ void best_match(params *v, seq_frag *frag1, seq_frag *frag2,
 }
 
 
-
 void replace_string(char **s1, char **s2, char c) 
 {
     int n;
@@ -323,6 +389,7 @@ void make_match_list(params *v, GPtrArray **matchlist)
                 continue;
             best_match(v,frag1,frag2,&(onematch->n1),&(onematch->n2),
                        &(onematch->l),&(onematch->p));
+	//printf("\n n1 = %d, n2 = %d, l= %d, p=%f\n", onematch->n1, onematch->n2, onematch->l, onematch->p);
             if (onematch->l > 0) {
                 onematch->frag1 = frag1;
                 onematch->frag2 = frag2;
